@@ -1,5 +1,5 @@
 
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -40,7 +40,7 @@ Expression *Expression::implicitCastTo(Scope *sc, Type *t)
             if (e->op == TOKint64)
                 return e->implicitCastTo(sc, t);
 
-                if (tyfrom == Tint32 &&
+            if (tyfrom == Tint32 &&
                 (op == TOKadd || op == TOKmin ||
                  op == TOKand || op == TOKor || op == TOKxor)
                )
@@ -55,10 +55,9 @@ Expression *Expression::implicitCastTo(Scope *sc, Type *t)
             }
             else
             {
-
-            warning("implicit conversion of expression (%s) of type %s to %s can cause loss of data",
-                toChars(), type->toChars(), t->toChars());
-                }
+                warning("implicit conversion of expression (%s) of type %s to %s can cause loss of data",
+                    toChars(), type->toChars(), t->toChars());
+            }
         }
 #if DMDV2
         if (match == MATCHconst && t == type->constOf())
@@ -123,7 +122,7 @@ MATCH Expression::implicitConvTo(Type *t)
     printf("Expression::implicitConvTo(this=%s, type=%s, t=%s)\n",
         toChars(), type->toChars(), t->toChars());
 #endif
-//static int nest; if (++nest == 10) halt();
+    //static int nest; if (++nest == 10) halt();
     if (!type)
     {   error("%s is not an expression", toChars());
         type = Type::terror;
@@ -283,25 +282,18 @@ MATCH IntegerExp::implicitConvTo(Type *t)
         case Tfloat64:
         case Tfloat80:
         {
-            real_t::MyMode mode;
             real_t f;
-            switch (toty)
-            {
-            case Tfloat32: mode = real_t::Float; break;
-            case Tfloat64: mode = real_t::Double; break;
-            case Tfloat80: mode = real_t::LongDouble; break;
-            }
             if (type->isunsigned())
             {
                 f = real_t((d_uns64) value);
-                f = f.convert(mode);
+                f = f.convert(t);
                 if ((d_uns64) f.toInt() != (d_uns64) value)
                     goto Lno;
             }
             else
             {
                 f = real_t((d_int64) value);
-                f = f.convert(mode);
+                f = f.convert(t);
                 if ((d_int64) f.toInt() != (d_int64) value)
                     goto Lno;
             }
@@ -409,8 +401,8 @@ MATCH StructLiteralExp::implicitConvTo(Type *t)
         ((TypeStruct *)type)->sym == ((TypeStruct *)t)->sym)
     {
         m = MATCHconst;
-        for (int i = 0; i < elements->dim; i++)
-        {   Expression *e = (Expression *)elements->data[i];
+        for (size_t i = 0; i < elements->dim; i++)
+        {   Expression *e = (*elements)[i];
             Type *te = e->type;
             if (t->mod == 0)
                 te = te->mutableOf();
@@ -426,21 +418,19 @@ MATCH StructLiteralExp::implicitConvTo(Type *t)
     }
     return m;
 }
-#endif //V2
+#endif
 
 MATCH StringExp::implicitConvTo(Type *t)
-{   MATCH m;
-
+{
 #if 0
     printf("StringExp::implicitConvTo(this=%s, committed=%d, type=%s, t=%s)\n",
         toChars(), committed, type->toChars(), t->toChars());
 #endif
-    if (!committed)
-    {
     if (!committed && t->ty == Tpointer && t->next->ty == Tvoid)
     {
         return MATCHnomatch;
     }
+    if (!committed)
     if (type->ty == Tsarray || type->ty == Tarray || type->ty == Tpointer)
     {
         if (type->next->ty == Tchar)
@@ -459,15 +449,16 @@ MATCH StringExp::implicitConvTo(Type *t)
                 L1:
                     if (t->next->ty == Tchar)
                         return MATCHexact;
-                    else if (t->next->ty == Twchar)
-                        return MATCHexact;
-                    else if (t->next->ty == Tdchar)
-                        return MATCHexact;
+                    else if (!committed)
+                    {   if (t->next->ty == Twchar)
+                            return MATCHexact;
+                        else if (t->next->ty == Tdchar)
+                            return MATCHexact;
+                    }
                     break;
             }
         }
     }
-    }
     return Expression::implicitConvTo(t);
 #if 0
     m = (MATCH)type->implicitConvTo(t);
@@ -484,10 +475,9 @@ MATCH ArrayLiteralExp::implicitConvTo(Type *t)
 {   MATCH result = MATCHexact;
 
 #if 0
-     printf("ArrayLiteralExp::implicitConvTo(this=%s, type=%s, t=%s)\n",
+    printf("ArrayLiteralExp::implicitConvTo(this=%s, type=%s, t=%s)\n",
         toChars(), type->toChars(), t->toChars());
 #endif
-
     Type *typeb = type->toBasetype();
     Type *tb = t->toBasetype();
     if ((tb->ty == Tarray || tb->ty == Tsarray) &&
@@ -499,8 +489,8 @@ MATCH ArrayLiteralExp::implicitConvTo(Type *t)
                 result = MATCHnomatch;
         }
 
-        for (int i = 0; i < elements->dim; i++)
-        {   Expression *e = (Expression *)elements->data[i];
+        for (size_t i = 0; i < elements->dim; i++)
+        {   Expression *e = (*elements)[i];
             MATCH m = (MATCH)e->implicitConvTo(tb->nextOf());
             if (m < result)
                 result = m;                     // remember worst match
@@ -614,7 +604,6 @@ MATCH DelegateExp::implicitConvTo(Type *t)
     if (result == 0)
     {
         // Look for pointers to functions where the functions are overloaded.
-        FuncDeclaration *f;
 
         t = t->toBasetype();
         if (type->ty == Tdelegate && type->nextOf()->ty == Tfunction &&
@@ -696,6 +685,12 @@ Expression *Expression::castTo(Scope *sc, Type *t)
 }
 
 
+Expression *ErrorExp::castTo(Scope *sc, Type *t)
+{
+    return this;
+}
+
+
 Expression *RealExp::castTo(Scope *sc, Type *t)
 {   Expression *e = this;
     if (type != t)
@@ -778,8 +773,6 @@ Expression *StringExp::castTo(Scope *sc, Type *t)
      * will result in a copy.
      * The this->string member is considered immutable.
      */
-    StringExp *se;
-    Type *tb;
     int copied = 0;
 
     //printf("StringExp::castTo(t = %s), '%s' committed = %d\n", t->toChars(), toChars(), committed);
@@ -790,7 +783,7 @@ Expression *StringExp::castTo(Scope *sc, Type *t)
         return new ErrorExp();
     }
 
-    se = this;
+    StringExp *se = this;
     if (!committed)
     {   se = (StringExp *)copy();
         se->committed = 1;
@@ -802,7 +795,7 @@ Expression *StringExp::castTo(Scope *sc, Type *t)
         return se;
     }
 
-    tb = t->toBasetype();
+    Type *tb = t->toBasetype();
     //printf("\ttype = %s\n", type->toChars());
     if (tb->ty == Tdelegate && type->toBasetype()->ty != Tdelegate)
         return Expression::castTo(sc, t);
@@ -946,7 +939,11 @@ Expression *StringExp::castTo(Scope *sc, Type *t)
             }
             se->string = buffer.extractData();
             se->len = newlen;
-            se->sz = tb->nextOf()->size();
+            {
+                d_uns64 szx = tb->nextOf()->size();
+                assert(szx <= 255);
+                se->sz = (unsigned char)szx;
+            }
             break;
 
         default:
@@ -961,9 +958,9 @@ L2:
     // See if need to truncate or extend the literal
     if (tb->ty == Tsarray)
     {
-        int dim2 = ((TypeSArray *)tb)->dim->toInteger();
+        dinteger_t dim2 = ((TypeSArray *)tb)->dim->toInteger();
 
-        //printf("dim from = %d, to = %d\n", se->len, dim2);
+        //printf("dim from = %d, to = %d\n", (int)se->len, (int)dim2);
 
         // Changing dimensions
         if (dim2 != se->len)
@@ -1071,10 +1068,10 @@ Expression *ArrayLiteralExp::castTo(Scope *sc, Type *t)
 
         e = (ArrayLiteralExp *)copy();
         e->elements = (Expressions *)elements->copy();
-        for (int i = 0; i < elements->dim; i++)
-        {   Expression *ex = (Expression *)elements->data[i];
+        for (size_t i = 0; i < elements->dim; i++)
+        {   Expression *ex = (*elements)[i];
             ex = ex->castTo(sc, tb->nextOf());
-            e->elements->data[i] = (void *)ex;
+            (*e->elements)[i] = ex;
         }
         e->type = t;
         return e;
@@ -1114,11 +1111,9 @@ Expression *AssocArrayLiteralExp::castTo(Scope *sc, Type *t)
         e->type = t;
         return e;
     }
-L1:
     return e->Expression::castTo(sc, t);
 }
 
-
 Expression *SymOffExp::castTo(Scope *sc, Type *t)
 {
 #if 0
@@ -1168,8 +1163,8 @@ Expression *SymOffExp::castTo(Scope *sc, Type *t)
                     else
 #endif
                     {
-                    e = new SymOffExp(loc, f, 0);
-                    e->type = t;
+                        e = new SymOffExp(loc, f, 0);
+                        e->type = t;
                     }
 #if DMDV2
                     f->tookAddressOf++;
@@ -1182,7 +1177,7 @@ Expression *SymOffExp::castTo(Scope *sc, Type *t)
     }
     else
     {
-    e->type = t;
+        e->type = t;
     }
     return e;
 }
@@ -1211,7 +1206,7 @@ Expression *DelegateExp::castTo(Scope *sc, Type *t)
             {
                 f = func->overloadExactMatch(tb->next);
                 if (f)
-                {   target_ptrdiff_t offset;
+                {   int offset;
                     if (f->tintro && f->tintro->next->isBaseOf(f->type->next, &offset) && offset)
                         error("%s", msg);
                     e = new DelegateExp(loc, e1, f);
@@ -1225,7 +1220,7 @@ Expression *DelegateExp::castTo(Scope *sc, Type *t)
         e = Expression::castTo(sc, t);
     }
     else
-    {   target_ptrdiff_t offset;
+    {   int offset;
 
         if (func->tintro && func->tintro->next->isBaseOf(func->type->next, &offset) && offset)
             error("%s", msg);
@@ -1434,7 +1429,7 @@ Expression *BinExp::typeCombine(Scope *sc)
         else if (t1n->ty == Tclass && t2n->ty == Tclass)
         {   ClassDeclaration *cd1 = t1n->isClassHandle();
             ClassDeclaration *cd2 = t2n->isClassHandle();
-            target_ptrdiff_t offset;
+            int offset;
 
             if (cd1->isBaseOf(cd2, &offset))
             {
@@ -1455,14 +1450,14 @@ Expression *BinExp::typeCombine(Scope *sc)
     }
     else if ((t1->ty == Tsarray || t1->ty == Tarray) &&
              e2->op == TOKnull && t2->ty == Tpointer && t2->nextOf()->ty == Tvoid)
-     {  /*  (T[n] op void*)
+    {   /*  (T[n] op void*)
          *  (T[] op void*)
          */
         goto Lx1;
     }
     else if ((t2->ty == Tsarray || t2->ty == Tarray) &&
              e1->op == TOKnull && t1->ty == Tpointer && t1->nextOf()->ty == Tvoid)
-     {  /*  (void* op T[n])
+    {   /*  (void* op T[n])
          *  (void* op T[])
          */
         goto Lx2;
@@ -1553,13 +1548,13 @@ Expression *BinExp::typeCombine(Scope *sc)
         assert(0);
     }
     else if (e1->isArrayOperand() && t1->ty == Tarray &&
-            e2->implicitConvTo(t1->nextOf()))
+             e2->implicitConvTo(t1->nextOf()))
     {   // T[] op T
         e2 = e2->castTo(sc, t1->nextOf());
         t = t1->nextOf()->arrayOf();
     }
     else if (e2->isArrayOperand() && t2->ty == Tarray &&
-            e1->implicitConvTo(t2->nextOf()))
+             e1->implicitConvTo(t2->nextOf()))
     {   // T op T[]
         e1 = e1->castTo(sc, t2->nextOf());
         t = t2->nextOf()->arrayOf();
@@ -1609,15 +1604,14 @@ Lt2:
 
 Expression *Expression::integralPromotions(Scope *sc)
 {
-     Expression *e = this;
+    Expression *e = this;
 
-     //printf("integralPromotions %s %s\n", e->toChars(), e->type->toChars());
+    //printf("integralPromotions %s %s\n", e->toChars(), e->type->toChars());
     switch (type->toBasetype()->ty)
     {
         case Tvoid:
             error("void has no value");
             return new ErrorExp();
-            break;
 
         case Tint8:
         case Tuns8:
