@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -40,7 +40,7 @@ Initializer *Initializer::syntaxCopy()
     return this;
 }
 
-Initializer *Initializer::semantic(Scope *sc, Type *t)
+Initializer *Initializer::semantic(Scope *sc, Type *t, int needInterpret)
 {
     return this;
 }
@@ -58,7 +58,7 @@ Initializers *Initializer::arraySyntaxCopy(Initializers *ai)
     {
         a = new Initializers();
         a->setDim(ai->dim);
-        for (int i = 0; i < a->dim; i++)
+        for (size_t i = 0; i < a->dim; i++)
         {   Initializer *e = (Initializer *)ai->data[i];
 
             e = e->syntaxCopy();
@@ -93,7 +93,7 @@ Initializer *VoidInitializer::syntaxCopy()
 }
 
 
-Initializer *VoidInitializer::semantic(Scope *sc, Type *t)
+Initializer *VoidInitializer::semantic(Scope *sc, Type *t, int needInterpret)
 {
     //printf("VoidInitializer::semantic(t = %p)\n", t);
     type = t;
@@ -129,7 +129,7 @@ Initializer *StructInitializer::syntaxCopy()
     assert(field.dim == value.dim);
     ai->field.setDim(field.dim);
     ai->value.setDim(value.dim);
-    for (int i = 0; i < field.dim; i++)
+    for (size_t i = 0; i < field.dim; i++)
     {
         ai->field.data[i] = field.data[i];
 
@@ -147,21 +147,25 @@ void StructInitializer::addInit(Identifier *field, Initializer *value)
     this->value.push(value);
 }
 
-Initializer *StructInitializer::semantic(Scope *sc, Type *t)
+Initializer *StructInitializer::semantic(Scope *sc, Type *t, int needInterpret)
 {
-    TypeStruct *ts;
     int errors = 0;
 
     //printf("StructInitializer::semantic(t = %s) %s\n", t->toChars(), toChars());
     vars.setDim(field.dim);
     t = t->toBasetype();
     if (t->ty == Tstruct)
-    {   unsigned i;
+    {
         unsigned fieldi = 0;
 
-        ts = (TypeStruct *)t;
+        TypeStruct *ts = (TypeStruct *)t;
         ad = ts->sym;
-        for (i = 0; i < field.dim; i++)
+        size_t nfields = ad->fields.dim;
+#if DMDV2
+        if (((StructDeclaration *)ad)->isnested)
+            nfields--;          // don't count pointer to outer
+#endif
+        for (size_t i = 0; i < field.dim; i++)
         {
             Identifier *id = (Identifier *)field.data[i];
             Initializer *val = (Initializer *)value.data[i];
@@ -170,8 +174,9 @@ Initializer *StructInitializer::semantic(Scope *sc, Type *t)
 
             if (id == NULL)
             {
-                if (fieldi >= ad->fields.dim)
+                if (fieldi >= nfields)
                 {   error(loc, "too many initializers for %s", ad->toChars());
+                    errors = 1;
                     field.remove(i);
                     i--;
                     continue;
@@ -188,15 +193,18 @@ Initializer *StructInitializer::semantic(Scope *sc, Type *t)
                 if (!s)
                 {
                     error(loc, "'%s' is not a member of '%s'", id->toChars(), t->toChars());
+                    errors = 1;
                     continue;
                 }
 
                 // Find out which field index it is
                 for (fieldi = 0; 1; fieldi++)
                 {
-                    if (fieldi >= ad->fields.dim)
+                    if (fieldi >= nfields)
                     {
-                        s->error("is not a per-instance initializable field");
+                        error(loc, "%s.%s is not a per-instance initializable field",
+                            t->toChars(), s->toChars());
+                        errors = 1;
                         break;
                     }
                     if (s == (Dsymbol *)ad->fields.data[fieldi])
@@ -205,7 +213,7 @@ Initializer *StructInitializer::semantic(Scope *sc, Type *t)
             }
             if (s && (v = s->isVarDeclaration()) != NULL)
             {
-                val = val->semantic(sc, v->type);
+                val = val->semantic(sc, v->type, needInterpret);
                 value.data[i] = (void *)val;
                 vars.data[i] = (void *)v;
             }
@@ -226,7 +234,7 @@ Initializer *StructInitializer::semantic(Scope *sc, Type *t)
         fd->endloc = loc;
         Expression *e = new FuncExp(loc, fd);
         ExpInitializer *ie = new ExpInitializer(loc, e);
-        return ie->semantic(sc, t);
+        return ie->semantic(sc, t, needInterpret);
     }
     else
     {
@@ -242,7 +250,6 @@ Initializer *StructInitializer::semantic(Scope *sc, Type *t)
     return this;
 }
 
-
 /***************************************
  * This works by transforming a struct initializer into
  * a struct literal. In the future, the two should be the
@@ -260,17 +267,96 @@ Expression *StructInitializer::toExpression()
     if (!sd)
         return NULL;
     Expressions *elements = new Expressions();
+    size_t nfields = ad->fields.dim;
+#if DMDV2
+    if (sd->isnested)
+       nfields--;
+#endif
+    elements->setDim(nfields);
+    for (size_t i = 0; i < elements->dim; i++)
+    {
+        elements->data[i] = NULL;
+    }
+    unsigned fieldi = 0;
     for (size_t i = 0; i < value.dim; i++)
     {
-        if (field.data[i])
+        Identifier *id = (Identifier *)field.data[i];
+        if (id)
+        {
+            Dsymbol * s = ad->search(loc, id, 0);
+            if (!s)
+            {
+                error(loc, "'%s' is not a member of '%s'", id->toChars(), sd->toChars());
+                goto Lno;
+            }
+
+            // Find out which field index it is
+            for (fieldi = 0; 1; fieldi++)
+            {
+                if (fieldi >= nfields)
+                {
+                    s->error("is not a per-instance initializable field");
+                    goto Lno;
+                }
+                if (s == (Dsymbol *)ad->fields.data[fieldi])
+                    break;
+            }
+        }
+        else if (fieldi >= nfields)
+        {   error(loc, "too many initializers for '%s'", ad->toChars());
             goto Lno;
+        }
         Initializer *iz = (Initializer *)value.data[i];
         if (!iz)
             goto Lno;
         Expression *ex = iz->toExpression();
         if (!ex)
             goto Lno;
-        elements->push(ex);
+        if (elements->data[fieldi])
+        {   error(loc, "duplicate initializer for field '%s'",
+                ((Dsymbol *)ad->fields.data[fieldi])->toChars());
+            goto Lno;
+        }
+        elements->data[fieldi] = ex;
+        ++fieldi;
+    }
+    // Now, fill in any missing elements with default initializers.
+    // We also need to validate any anonymous unions
+    for (size_t i = 0; i < elements->dim; )
+    {
+        VarDeclaration * vd = ((Dsymbol *)ad->fields.data[i])->isVarDeclaration();
+        int unionSize = ad->numFieldsInUnion(i);
+        if (unionSize == 1)
+        {   // Not a union -- default initialize if missing
+            if (!elements->data[i])
+                elements->data[i] = vd->type->defaultInit();
+        }
+        else
+        {   // anonymous union -- check for errors
+            int found = -1; // index of the first field with an initializer
+            for (int j = i; j < i + unionSize; ++j)
+            {
+                if (!elements->data[j])
+                    continue;
+                if (found >= 0)
+                {
+                    VarDeclaration * v1 = ((Dsymbol *)ad->fields.data[found])->isVarDeclaration();
+                    VarDeclaration * v = ((Dsymbol *)ad->fields.data[j])->isVarDeclaration();
+                    error(loc, "%s cannot have initializers for fields %s and %s in same union",
+                        ad->toChars(),
+                        v1->toChars(), v->toChars());
+                    goto Lno;
+                }
+                found = j;
+            }
+            if (found == -1)
+            {
+                error(loc, "no initializer for union that contains field %s",
+                    vd->toChars());
+                goto Lno;
+            }
+        }
+        i += unionSize;
     }
     e = new StructLiteralExp(loc, sd, elements);
     e->type = sd->type;
@@ -278,7 +364,6 @@ Expression *StructInitializer::toExpression()
 
 Lno:
     delete elements;
-    //error(loc, "struct initializers as expressions are not allowed");
     return NULL;
 }
 
@@ -287,7 +372,7 @@ void StructInitializer::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     //printf("StructInitializer::toCBuffer()\n");
     buf->writebyte('{');
-    for (int i = 0; i < field.dim; i++)
+    for (size_t i = 0; i < field.dim; i++)
     {
         if (i > 0)
             buf->writebyte(',');
@@ -323,7 +408,7 @@ Initializer *ArrayInitializer::syntaxCopy()
     assert(index.dim == value.dim);
     ai->index.setDim(index.dim);
     ai->value.setDim(value.dim);
-    for (int i = 0; i < ai->value.dim; i++)
+    for (size_t i = 0; i < ai->value.dim; i++)
     {   Expression *e = (Expression *)index.data[i];
         if (e)
             e = e->syntaxCopy();
@@ -344,10 +429,10 @@ void ArrayInitializer::addInit(Expression *index, Initializer *value)
     type = NULL;
 }
 
-Initializer *ArrayInitializer::semantic(Scope *sc, Type *t)
+Initializer *ArrayInitializer::semantic(Scope *sc, Type *t, int needInterpret)
 {   unsigned i;
     unsigned length;
-    const unsigned amax = 0x80000000;
+    const uintmax_t amax = 0x80000000;
 
     //printf("ArrayInitializer::semantic(%s)\n", t->toChars());
     if (sem)                            // if semantic() already run
@@ -379,7 +464,7 @@ Initializer *ArrayInitializer::semantic(Scope *sc, Type *t)
         }
 
         Initializer *val = (Initializer *)value.data[i];
-        val = val->semantic(sc, t->nextOf());
+        val = val->semantic(sc, t->nextOf(), needInterpret);
         value.data[i] = (void *)val;
         length++;
         if (length == 0)
@@ -404,7 +489,7 @@ Initializer *ArrayInitializer::semantic(Scope *sc, Type *t)
         goto Lerr;
     }
     return this;
-    
+
 Lerr:
     return new ExpInitializer(loc, new ErrorExp());
 }
@@ -415,7 +500,6 @@ Lerr:
 
 Expression *ArrayInitializer::toExpression()
 {   Expressions *elements;
-    Expression *e;
 
     //printf("ArrayInitializer::toExpression(), dim = %d\n", dim);
     //static int i; if (++i == 2) halt();
@@ -470,8 +554,8 @@ Expression *ArrayInitializer::toExpression()
         if (!ex)
         {
             goto Lno;
-    }
-    elements->data[j] = ex;
+        }
+        elements->data[j] = ex;
     }
 
     /* Fill in any missing elements with the default initializer
@@ -491,7 +575,6 @@ Expression *ArrayInitializer::toExpression()
     }
 
     Expression *e = new ArrayLiteralExp(loc, elements);
-
     e->type = type;
     return e;
     }
@@ -507,15 +590,14 @@ Lno:
  */
 
 Initializer *ArrayInitializer::toAssocArrayInitializer()
-{   Expressions *keys;
-    Expressions *values;
+{
     Expression *e;
 
     //printf("ArrayInitializer::toAssocArrayInitializer()\n");
     //static int i; if (++i == 2) halt();
-    keys = new Expressions();
+    Expressions *keys = new Expressions();
     keys->setDim(value.dim);
-    values = new Expressions();
+    Expressions *values = new Expressions();
     values->setDim(value.dim);
 
     for (size_t i = 0; i < value.dim; i++)
@@ -583,7 +665,7 @@ Laa:
 void ArrayInitializer::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     buf->writebyte('[');
-    for (int i = 0; i < index.dim; i++)
+    for (size_t i = 0; i < index.dim; i++)
     {
         if (i > 0)
             buf->writebyte(',');
@@ -614,10 +696,16 @@ Initializer *ExpInitializer::syntaxCopy()
     return new ExpInitializer(loc, exp->syntaxCopy());
 }
 
-Initializer *ExpInitializer::semantic(Scope *sc, Type *t)
+Initializer *ExpInitializer::semantic(Scope *sc, Type *t, int needInterpret)
 {
     //printf("ExpInitializer::semantic(%s), type = %s\n", exp->toChars(), t->toChars());
     exp = exp->semantic(sc);
+    int wantOptimize = needInterpret ? WANTinterpret|WANTvalue : WANTvalue;
+
+    int olderrors = global.errors;
+    exp = exp->optimize(wantOptimize);
+    if (!global.gag && olderrors != global.errors)
+        return this; // Failed, suppress duplicate error messages
     Type *tb = t->toBasetype();
 
     /* Look for case of initializing a static array with a too-short
@@ -650,7 +738,7 @@ Initializer *ExpInitializer::semantic(Scope *sc, Type *t)
 
     exp = exp->implicitCastTo(sc, t);
 L1:
-    exp = exp->optimize(WANTvalue | WANTinterpret);
+    exp = exp->optimize(wantOptimize);
     //printf("-ExpInitializer::semantic(): "); exp->print();
     return this;
 }
