@@ -1,5 +1,5 @@
 
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -31,6 +31,7 @@
 #include "id.h"
 #include "import.h"
 #include "template.h"
+
 #include "rmem.h"
 
 #ifndef IN_GCC
@@ -53,46 +54,6 @@
 
 void obj_lzext(Symbol *s1,Symbol *s2);
 
-#ifdef IN_GCC
-#if 0
-static bool m_in_a(Module * m, Array * a) {
-    for (unsigned i = 0; i < a->dim; i++)
-        if ( m == (Module *) a->data[i] )
-            return true;
-    return false;
-}
-static void find_module_deps(Module * the_module, Array * out_deps)
-{
-    Array work;
-    unsigned wi = 0;
-
-    work.push(the_module);
-    while (wi < work.dim) {
-        Module * a_module = (Module *) work.data[wi];
-        for (unsigned i = 0; i < a_module->aimports.dim; i++) {
-            Module * an_imp = (Module*) a_module->aimports.data[i];
-            if (! an_imp->needModuleInfo() ||
-                m_in_a(an_imp, & work) || m_in_a(an_imp, out_deps))
-                continue;
-            if (an_imp->strictlyneedmoduleinfo)
-            {
-                out_deps->push(an_imp);
-                fprintf(stderr, "idep: %s -> %s", the_module->toPrettyChars(),
-                    an_imp->toPrettyChars());
-                if (a_module != the_module)
-                    fprintf(stderr, " (via %s)", a_module->toPrettyChars());
-                fprintf(stderr, "\n");
-            }
-            else
-                work.push(an_imp);
-        }
-        wi++;
-    }
-}
-#endif
-#endif
-
-
 /* ================================================================== */
 
 // Put out instance of ModuleInfo for this Module
@@ -102,7 +63,6 @@ void Module::genmoduleinfo()
     //printf("Module::genmoduleinfo() %s\n", toChars());
 
     Symbol *msym = toSymbol();
-    unsigned offset;
 #if DMDV2
     unsigned sizeof_ModuleInfo = 18 * PTRSIZE;
 #else
@@ -135,27 +95,20 @@ void Module::genmoduleinfo()
         dtxoff(&dt, moduleinfo->toVtblSymbol(), 0, TYnptr); // vtbl for ModuleInfo
     else
     {   //printf("moduleinfo is null\n");
-        dtdword(&dt, 0);                // BUG: should be an assert()
+        dtsize_t(&dt, 0);                // BUG: should be an assert()
     }
-    dtdword(&dt, 0);                    // monitor
-
-#if DMDV2
-    FuncDeclaration *sgetmembers = findGetMembers();
-    if (sgetmembers)
-        dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
-    else
-#endif
+    dtsize_t(&dt, 0);                    // monitor
 
     // name[]
     const char *name = toPrettyChars();
     size_t namelen = strlen(name);
-    dtdword(&dt, namelen);
+    dtsize_t(&dt, namelen);
     dtabytes(&dt, TYnptr, 0, namelen + 1, name);
 
     ClassDeclarations aclasses;
 
     //printf("members->dim = %d\n", members->dim);
-    for (int i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {   Dsymbol *member = (Dsymbol *)members->data[i];
 
         //printf("\tmember '%s'\n", member->toChars());
@@ -164,105 +117,84 @@ void Module::genmoduleinfo()
 
     // importedModules[]
     int aimports_dim = aimports.dim;
-#ifdef IN_GCC
-#if 0
-    Array adeps;
-
-    if (! d_gcc_supports_weak())
-    {
-        find_module_deps(this, & adeps);
-        aimports_dim = adeps.dim;
-    }
-    else
-#endif
-#endif
-    for (int i = 0; i < aimports.dim; i++)
+    for (size_t i = 0; i < aimports.dim; i++)
     {   Module *m = (Module *)aimports.data[i];
-        if (!m->needModuleInfo())
+        if (!m->needmoduleinfo)
             aimports_dim--;
     }
-    dtdword(&dt, aimports_dim);
+    dtsize_t(&dt, aimports_dim);
     if (aimports_dim)
         dtxoff(&dt, csym, sizeof_ModuleInfo, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // localClasses[]
-    dtdword(&dt, aclasses.dim);
+    dtsize_t(&dt, aclasses.dim);
     if (aclasses.dim)
         dtxoff(&dt, csym, sizeof_ModuleInfo + aimports_dim * PTRSIZE, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     if (needmoduleinfo)
-        dti32(&dt, 8|0, true);          // flags (4 means MIstandalone)
+        dtsize_t(&dt, 8|0);              // flags (4 means MIstandalone)
     else
-        dti32(&dt, 8|4, true);          // flags (4 means MIstandalone)
+        dtsize_t(&dt, 8|4);              // flags (4 means MIstandalone)
 
     if (sctor)
         dtxoff(&dt, sctor, 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     if (sdtor)
         dtxoff(&dt, sdtor, 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     if (stest)
         dtxoff(&dt, stest, 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
-    dtdword(&dt, 0);                    // xgetMembers
+#if DMDV2
+    FuncDeclaration *sgetmembers = findGetMembers();
+    if (sgetmembers)
+        dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
+    else
+#endif
+        dtsize_t(&dt, 0);                        // xgetMembers
 
     if (sictor)
         dtxoff(&dt, sictor, 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
 #if DMDV2
     // void*[4] reserved;
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);
 #endif
     //////////////////////////////////////////////
 
-#ifdef IN_GCC
-#if 0
-    if (! d_gcc_supports_weak())
-        for (i = 0; i < adeps.dim; i++)
-        {
-            Module *m;
-            Symbol *s;
-
-            m = (Module *) adeps.data[i];
-            s = m->toSymbol();
-            s->Sflags |= SFLweak; // doesn't do anything yet, but see d-decls.cc:Module::toSymbol
-            dtxoff(&dt, s, 0, TYnptr);
-        }
-    else
-#endif
-#endif
-   for (int i = 0; i < aimports.dim; i++)
+    for (size_t i = 0; i < aimports.dim; i++)
     {   Module *m = (Module *)aimports.data[i];
 
-        if (m->needModuleInfo())
+        if (m->needmoduleinfo)
         {   Symbol *s = m->toSymbol();
+
             /* Weak references don't pull objects in from the library,
              * they resolve to 0 if not pulled in by something else.
              * Don't pull in a module just because it was imported.
              */
 #if !OMFOBJ // Optlink crashes with weak symbols at EIP 41AFE7, 402000
-     s->Sflags |= SFLweak;
+            s->Sflags |= SFLweak;
 #endif
             dtxoff(&dt, s, 0, TYnptr);
         }
     }
 
-    for (int i = 0; i < aclasses.dim; i++)
+    for (size_t i = 0; i < aclasses.dim; i++)
     {
         ClassDeclaration *cd = (ClassDeclaration *)aclasses.data[i];
         dtxoff(&dt, cd->toSymbol(), 0, TYnptr);
@@ -309,10 +241,11 @@ void Dsymbol::toObjFile(int multiobj)
     }
 #endif
 }
+
 /* ================================================================== */
 
 void ClassDeclaration::toObjFile(int multiobj)
-{   unsigned i;
+{
     unsigned offset;
     Symbol *sinit;
     enum_SC scclass;
@@ -322,6 +255,11 @@ void ClassDeclaration::toObjFile(int multiobj)
     if (!members)
         return;
 
+    if (multiobj)
+    {   obj_append(this);
+        return;
+    }
+
     if (global.params.symdebug)
         toDebug();
 
@@ -332,7 +270,7 @@ void ClassDeclaration::toObjFile(int multiobj)
         scclass = SCcomdat;
 
     // Put out the members
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {
         Dsymbol *member;
 
@@ -371,7 +309,7 @@ void ClassDeclaration::toObjFile(int multiobj)
 
             // Call each of the destructors in dtors[]
             // in reverse order
-            for (i = 0; i < dtors.dim; i++)
+            for (size_t i = 0; i < dtors.dim; i++)
             {   DtorDeclaration *d = (DtorDeclaration *)dtors.data[i];
                 Symbol *s = d->toSymbol();
                 elem *e = el_bin(OPcall, TYvoid, el_var(s), el_var(sthis));
@@ -450,11 +388,15 @@ void ClassDeclaration::toObjFile(int multiobj)
        }
      */
     dt_t *dt = NULL;
-    offset = CLASSINFO_SIZE;                    // must be ClassInfo.size
+    unsigned classinfo_size = global.params.is64bit ? CLASSINFO_SIZE_64 : CLASSINFO_SIZE;    // must be ClassInfo.size
+    offset = classinfo_size;
     if (classinfo)
     {
-        if (classinfo->structsize != CLASSINFO_SIZE)
+        if (classinfo->structsize != classinfo_size)
         {
+#ifdef DEBUG
+            printf("CLASSINFO_SIZE = x%x, classinfo->structsize = x%x\n", offset, classinfo->structsize);
+#endif
             error("mismatch between dmd and object.d or object.di found. Check installation and import paths with -v compiler switch.");
             fatal();
         }
@@ -463,12 +405,12 @@ void ClassDeclaration::toObjFile(int multiobj)
     if (classinfo)
         dtxoff(&dt, classinfo->toVtblSymbol(), 0, TYnptr); // vtbl for ClassInfo
     else
-        dtdword(&dt, 0);                // BUG: should be an assert()
-    dtdword(&dt, 0);                    // monitor
+        dtsize_t(&dt, 0);                // BUG: should be an assert()
+    dtsize_t(&dt, 0);                    // monitor
 
     // initializer[]
     assert(structsize >= 8);
-    dtdword(&dt, structsize);           // size
+    dtsize_t(&dt, structsize);           // size
     dtxoff(&dt, sinit, 0, TYnptr);      // initializer
 
     // name[]
@@ -478,37 +420,37 @@ void ClassDeclaration::toObjFile(int multiobj)
     {   name = toPrettyChars();
         namelen = strlen(name);
     }
-    dtdword(&dt, namelen);
+    dtsize_t(&dt, namelen);
     dtabytes(&dt, TYnptr, 0, namelen + 1, name);
 
     // vtbl[]
-    dtdword(&dt, vtbl.dim);
+    dtsize_t(&dt, vtbl.dim);
     dtxoff(&dt, vtblsym, 0, TYnptr);
 
     // interfaces[]
-    dtdword(&dt, vtblInterfaces->dim);
+    dtsize_t(&dt, vtblInterfaces->dim);
     if (vtblInterfaces->dim)
         dtxoff(&dt, csym, offset, TYnptr);      // (*)
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // base
     if (baseClass)
         dtxoff(&dt, baseClass->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // destructor
     if (dtor)
         dtxoff(&dt, dtor->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // invariant
     if (inv)
         dtxoff(&dt, inv->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // flags
     int flags = 4 | isCOMclass();
@@ -533,35 +475,35 @@ void ClassDeclaration::toObjFile(int multiobj)
     }
     flags |= 2;                 // no pointers
   L2:
-    dti32(&dt, flags, true);
+    dtsize_t(&dt, flags);
 
 
     // deallocator
     if (aggDelete)
         dtxoff(&dt, aggDelete->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
     // offTi[]
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);            // null for now, fix later
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);            // null for now, fix later
 
     // defaultConstructor
     if (defaultCtor)
         dtxoff(&dt, defaultCtor->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
 
 #if DMDV2
     FuncDeclaration *sgetmembers = findGetMembers();
     if (sgetmembers)
         dtxoff(&dt, sgetmembers->toSymbol(), 0, TYnptr);
     else
-        dtdword(&dt, 0);        // module getMembers() function
+        dtsize_t(&dt, 0);        // module getMembers() function
 #endif
 
     dtxoff(&dt, type->vtinfo->toSymbol(), 0, TYnptr);   // typeinfo
-    //dtdword(&dt, 0);
+    //dtsize_t(&dt, 0);
 
     //////////////////////////////////////////////
 
@@ -569,7 +511,7 @@ void ClassDeclaration::toObjFile(int multiobj)
     // of the fixup (*)
 
     offset += vtblInterfaces->dim * (4 * PTRSIZE);
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         ClassDeclaration *id = b->base;
 
@@ -587,10 +529,10 @@ void ClassDeclaration::toObjFile(int multiobj)
         dtxoff(&dt, id->toSymbol(), 0, TYnptr);         // ClassInfo
 
         // vtbl[]
-        dtdword(&dt, id->vtbl.dim);
+        dtsize_t(&dt, id->vtbl.dim);
         dtxoff(&dt, csym, offset, TYnptr);
 
-        dtdword(&dt, b->offset);                        // this offset
+        dtsize_t(&dt, b->offset);                        // this offset
 
         offset += id->vtbl.dim * PTRSIZE;
     }
@@ -598,27 +540,24 @@ void ClassDeclaration::toObjFile(int multiobj)
     // Put out the vtblInterfaces->data[].vtbl[]
     // This must be mirrored with ClassDeclaration::baseVtblOffset()
     //printf("putting out %d interface vtbl[]s for '%s'\n", vtblInterfaces->dim, toChars());
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         ClassDeclaration *id = b->base;
-        int j;
 
         //printf("    interface[%d] is '%s'\n", i, id->toChars());
-        j = 0;
+        size_t j = 0;
         if (id->vtblOffset())
         {
             // First entry is ClassInfo reference
             //dtxoff(&dt, id->toSymbol(), 0, TYnptr);
 
             // First entry is struct Interface reference
-            dtxoff(&dt, csym, CLASSINFO_SIZE + i * (4 * PTRSIZE), TYnptr);
+            dtxoff(&dt, csym, classinfo_size + i * (4 * PTRSIZE), TYnptr);
             j = 1;
         }
         assert(id->vtbl.dim == b->vtbl.dim);
         for (; j < id->vtbl.dim; j++)
         {
-            FuncDeclaration *fd;
-
             assert(j < b->vtbl.dim);
 #if 0
             Object *o = (Object *)b->vtbl.data[j];
@@ -630,11 +569,11 @@ void ClassDeclaration::toObjFile(int multiobj)
                 printf("s->kind() = '%s'\n", s->kind());
             }
 #endif
-            fd = (FuncDeclaration *)b->vtbl.data[j];
+            FuncDeclaration *fd = (FuncDeclaration *)b->vtbl.data[j];
             if (fd)
                 dtxoff(&dt, fd->toThunkSymbol(b->offset), 0, TYnptr);
             else
-                dtdword(&dt, 0);
+                dtsize_t(&dt, 0);
         }
     }
 
@@ -643,27 +582,26 @@ void ClassDeclaration::toObjFile(int multiobj)
     // This must be mirrored with ClassDeclaration::baseVtblOffset()
     //printf("putting out overriding interface vtbl[]s for '%s' at offset x%x\n", toChars(), offset);
     ClassDeclaration *cd;
-    Array bvtbl;
+    FuncDeclarations bvtbl;
 
     for (cd = this->baseClass; cd; cd = cd->baseClass)
     {
-        for (int k = 0; k < cd->vtblInterfaces->dim; k++)
+        for (size_t k = 0; k < cd->vtblInterfaces->dim; k++)
         {   BaseClass *bs = (BaseClass *)cd->vtblInterfaces->data[k];
 
             if (bs->fillVtbl(this, &bvtbl, 0))
             {
                 //printf("\toverriding vtbl[] for %s\n", bs->base->toChars());
                 ClassDeclaration *id = bs->base;
-                int j;
 
-                j = 0;
+                size_t j = 0;
                 if (id->vtblOffset())
                 {
                     // First entry is ClassInfo reference
                     //dtxoff(&dt, id->toSymbol(), 0, TYnptr);
 
                     // First entry is struct Interface reference
-                    dtxoff(&dt, cd->toSymbol(), CLASSINFO_SIZE + k * (4 * PTRSIZE), TYnptr);
+                    dtxoff(&dt, cd->toSymbol(), classinfo_size + k * (4 * PTRSIZE), TYnptr);
                     j = 1;
                 }
 
@@ -676,7 +614,7 @@ void ClassDeclaration::toObjFile(int multiobj)
                     if (fd)
                         dtxoff(&dt, fd->toThunkSymbol(bs->offset), 0, TYnptr);
                     else
-                        dtdword(&dt, 0);
+                        dtsize_t(&dt, 0);
                 }
             }
         }
@@ -686,42 +624,39 @@ void ClassDeclaration::toObjFile(int multiobj)
     // Put out the overriding interface vtbl[]s.
     // This must be mirrored with ClassDeclaration::baseVtblOffset()
     //printf("putting out overriding interface vtbl[]s for '%s' at offset x%x\n", toChars(), offset);
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         ClassDeclaration *cd;
 
         for (cd = this->baseClass; cd; cd = cd->baseClass)
         {
-            for (int k = 0; k < cd->vtblInterfaces->dim; k++)
+            for (size_t k = 0; k < cd->vtblInterfaces->dim; k++)
             {   BaseClass *bs = (BaseClass *)cd->vtblInterfaces->data[k];
 
                 if (b->base == bs->base)
                 {
                     //printf("\toverriding vtbl[] for %s\n", b->base->toChars());
                     ClassDeclaration *id = b->base;
-                    int j;
 
-                    j = 0;
+                    size_t j = 0;
                     if (id->vtblOffset())
                     {
                         // First entry is ClassInfo reference
                         //dtxoff(&dt, id->toSymbol(), 0, TYnptr);
 
                         // First entry is struct Interface reference
-                        dtxoff(&dt, cd->toSymbol(), CLASSINFO_SIZE + k * (4 * PTRSIZE), TYnptr);
+                        dtxoff(&dt, cd->toSymbol(), classinfo_size + k * (4 * PTRSIZE), TYnptr);
                         j = 1;
                     }
 
                     for (; j < id->vtbl.dim; j++)
                     {
-                        FuncDeclaration *fd;
-
                         assert(j < b->vtbl.dim);
-                        fd = (FuncDeclaration *)b->vtbl.data[j];
+                        FuncDeclaration *fd = (FuncDeclaration *)b->vtbl.data[j];
                         if (fd)
                             dtxoff(&dt, fd->toThunkSymbol(bs->offset), 0, TYnptr);
                         else
-                            dtdword(&dt, 0);
+                            dtsize_t(&dt, 0);
                     }
                 }
             }
@@ -744,13 +679,8 @@ void ClassDeclaration::toObjFile(int multiobj)
     // Put out the vtbl[]
     //printf("putting out %s.vtbl[]\n", toChars());
     dt = NULL;
-    if (0)
-        i = 0;
-    else
-    {   dtxoff(&dt, csym, 0, TYnptr);           // first entry is ClassInfo reference
-        i = 1;
-    }
-    for (; i < vtbl.dim; i++)
+    dtxoff(&dt, csym, 0, TYnptr);           // first entry is ClassInfo reference
+    for (size_t i = 1; i < vtbl.dim; i++)
     {
         FuncDeclaration *fd = ((Dsymbol *)vtbl.data[i])->isFuncDeclaration();
 
@@ -764,7 +694,7 @@ void ClassDeclaration::toObjFile(int multiobj)
                  * If fd overlaps with any function in the vtbl[], then
                  * issue 'hidden' error.
                  */
-                for (int j = 1; j < vtbl.dim; j++)
+                for (size_t j = 1; j < vtbl.dim; j++)
                 {   if (j == i)
                         continue;
                     FuncDeclaration *fd2 = ((Dsymbol *)vtbl.data[j])->isFuncDeclaration();
@@ -785,17 +715,17 @@ void ClassDeclaration::toObjFile(int multiobj)
                     }
                 }
             }
-#endif //V2
+#endif
             dtxoff(&dt, s, 0, TYnptr);
         }
         else
-            dtdword(&dt, 0);
+            dtsize_t(&dt, 0);
     }
     vtblsym->Sdt = dt;
     vtblsym->Sclass = scclass;
     vtblsym->Sfl = FLdata;
 #if ELFOBJ
-      vtblsym->Sseg = CDATA;
+    vtblsym->Sseg = CDATA;
 #endif
 #if MACHOBJ
     vtblsym->Sseg = DATA;
@@ -813,13 +743,12 @@ void ClassDeclaration::toObjFile(int multiobj)
 unsigned ClassDeclaration::baseVtblOffset(BaseClass *bc)
 {
     unsigned csymoffset;
-    int i;
 
     //printf("ClassDeclaration::baseVtblOffset('%s', bc = %p)\n", toChars(), bc);
-    csymoffset = CLASSINFO_SIZE;
+    csymoffset = global.params.is64bit ? CLASSINFO_SIZE_64 : CLASSINFO_SIZE;    // must be ClassInfo.size
     csymoffset += vtblInterfaces->dim * (4 * PTRSIZE);
 
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {
         BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
 
@@ -837,7 +766,7 @@ unsigned ClassDeclaration::baseVtblOffset(BaseClass *bc)
 
     for (cd = this->baseClass; cd; cd = cd->baseClass)
     {
-        for (int k = 0; k < cd->vtblInterfaces->dim; k++)
+        for (size_t k = 0; k < cd->vtblInterfaces->dim; k++)
         {   BaseClass *bs = (BaseClass *)cd->vtblInterfaces->data[k];
 
             if (bs->fillVtbl(this, NULL, 0))
@@ -852,14 +781,14 @@ unsigned ClassDeclaration::baseVtblOffset(BaseClass *bc)
     }
 #endif
 #if INTERFACE_VIRTUAL
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         ClassDeclaration *cd;
 
         for (cd = this->baseClass; cd; cd = cd->baseClass)
         {
             //printf("\tbase class %s\n", cd->toChars());
-            for (int k = 0; k < cd->vtblInterfaces->dim; k++)
+            for (size_t k = 0; k < cd->vtblInterfaces->dim; k++)
             {   BaseClass *bs = (BaseClass *)cd->vtblInterfaces->data[k];
 
                 if (bc == bs)
@@ -879,8 +808,7 @@ unsigned ClassDeclaration::baseVtblOffset(BaseClass *bc)
 /* ================================================================== */
 
 void InterfaceDeclaration::toObjFile(int multiobj)
-{   unsigned i;
-    unsigned offset;
+{
     Symbol *sinit;
     enum_SC scclass;
 
@@ -892,17 +820,13 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     if (global.params.symdebug)
         toDebug();
 
+    scclass = SCglobal;
     if (inTemplateInstance())
         scclass = SCcomdat;
-    else
-        scclass = SCglobal;
 
     // Put out the members
-    for (i = 0; i < members->dim; i++)
-    {
-        Dsymbol *member;
-
-        member = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *member = (Dsymbol *)members->data[i];
         if (!member->isFuncDeclaration())
             member->toObjFile(0);
     }
@@ -923,7 +847,7 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     csym->Sfl = FLdata;
 
     /* The layout is:
-        {
+       {
             void **vptr;
             monitor_t monitor;
             byte[] initializer;         // static initialization data
@@ -941,74 +865,77 @@ void InterfaceDeclaration::toObjFile(int multiobj)
             const(MemberInfo[]) function(string) xgetMembers;   // module getMembers() function
 #endif
             TypeInfo typeinfo;
-        }
+       }
      */
     dt_t *dt = NULL;
 
     if (classinfo)
         dtxoff(&dt, classinfo->toVtblSymbol(), 0, TYnptr); // vtbl for ClassInfo
     else
-        dtdword(&dt, 0);                // BUG: should be an assert()
-    dtdword(&dt, 0);                    // monitor
+        dtsize_t(&dt, 0);                // BUG: should be an assert()
+    dtsize_t(&dt, 0);                    // monitor
 
     // initializer[]
-    dtdword(&dt, 0);                    // size
-    dtdword(&dt, 0);                    // initializer
+    dtsize_t(&dt, 0);                    // size
+    dtsize_t(&dt, 0);                    // initializer
 
     // name[]
     const char *name = toPrettyChars();
     size_t namelen = strlen(name);
-    dtdword(&dt, namelen);
+    dtsize_t(&dt, namelen);
     dtabytes(&dt, TYnptr, 0, namelen + 1, name);
 
     // vtbl[]
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);
 
-    // vtblInterfaces->data[]
-    dtdword(&dt, vtblInterfaces->dim);
+    // vtblInterfaces[]
+    unsigned offset;
+    dtsize_t(&dt, vtblInterfaces->dim);
     if (vtblInterfaces->dim)
     {
+        offset = global.params.is64bit ? CLASSINFO_SIZE_64 : CLASSINFO_SIZE;    // must be ClassInfo.size
         if (classinfo)
         {
-            if (classinfo->structsize != CLASSINFO_SIZE)
+            if (classinfo->structsize != offset)
             {
                 error("mismatch between dmd and object.d or object.di found. Check installation and import paths with -v compiler switch.");
                 fatal();
             }
         }
-        offset = CLASSINFO_SIZE;
         dtxoff(&dt, csym, offset, TYnptr);      // (*)
     }
     else
-        dtdword(&dt, 0);
+    {   offset = 0;
+        dtsize_t(&dt, 0);
+    }
 
     // base
     assert(!baseClass);
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 
     // dtor
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 
     // invariant
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 
     // flags
-    dtdword(&dt, 4 | isCOMinterface() | 32); //dti32(&dt, 4 | isCOMinterface(), true);
+    dtsize_t(&dt, 4 | isCOMinterface() | 32);
 
     // deallocator
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 
     // offTi[]
-    dtdword(&dt, 0);
-    dtdword(&dt, 0);            // null for now, fix later
+    dtsize_t(&dt, 0);
+    dtsize_t(&dt, 0);            // null for now, fix later
 
     // defaultConstructor
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 
 #if DMDV2
     // xgetMembers
-    dtdword(&dt, 0);
+    dtsize_t(&dt, 0);
 #endif
 
     dtxoff(&dt, type->vtinfo->toSymbol(), 0, TYnptr);   // typeinfo
@@ -1019,7 +946,7 @@ void InterfaceDeclaration::toObjFile(int multiobj)
     // of the fixup (*)
 
     offset += vtblInterfaces->dim * (4 * PTRSIZE);
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         ClassDeclaration *id = b->base;
 
@@ -1027,16 +954,16 @@ void InterfaceDeclaration::toObjFile(int multiobj)
         dtxoff(&dt, id->toSymbol(), 0, TYnptr);
 
         // vtbl[]
-        dtdword(&dt, 0);
-        dtdword(&dt, 0);
+        dtsize_t(&dt, 0);
+        dtsize_t(&dt, 0);
 
         // this offset
-        dtdword(&dt, b->offset);
+        dtsize_t(&dt, b->offset);
     }
 
     csym->Sdt = dt;
 #if ELFOBJ
-      csym->Sseg = CDATA;
+    csym->Sseg = CDATA;
 #endif
 #if MACHOBJ
     csym->Sseg = DATA;
@@ -1189,7 +1116,6 @@ void VarDeclaration::toObjFile(int multiobj)
                         init && init->isExpInitializer())
                         return;
 #endif
-
                     s->Sclass = SCcomdat;
                     break;
                 }
@@ -1252,15 +1178,15 @@ void VarDeclaration::toObjFile(int multiobj)
         else
             s->Sseg = DATA;
 #endif
-        if (sz)
+        if (1 || sz)
         {   outdata(s);
             if (isExport())
                 obj_export(s,0);
         }
     }
 #ifdef IN_GCC
-    else {
-        // This is needed for VarDeclarations in mixins that are to be
+    else
+    {   // This is needed for VarDeclarations in mixins that are to be
         // local variables of a function.  Otherwise, it would be
         // enough to make a check for isVarDeclaration() in
         // DeclarationExp::toElem.
@@ -1287,7 +1213,7 @@ void TypedefDeclaration::toObjFile(int multiobj)
     {
         enum_SC scclass = SCglobal;
         if (inTemplateInstance())
-                scclass = SCcomdat;
+            scclass = SCcomdat;
 
         // Generate static initializer
         toInitializer();
@@ -1299,8 +1225,7 @@ void TypedefDeclaration::toObjFile(int multiobj)
 #if MACHOBJ
         sinit->Sseg = DATA;
 #endif
-        if (! sinit->Sdt)
-            sinit->Sdt = tc->sym->init->toDt();
+        sinit->Sdt = tc->sym->init->toDt();
         outdata(sinit);
     }
 }
@@ -1328,7 +1253,7 @@ void EnumDeclaration::toObjFile(int multiobj)
     {
         enum_SC scclass = SCglobal;
         if (inTemplateInstance())
-                scclass = SCcomdat;
+            scclass = SCcomdat;
 
         // Generate static initializer
         toInitializer();
@@ -1347,15 +1272,9 @@ void EnumDeclaration::toObjFile(int multiobj)
 #if DMDV2
         tc->sym->defaultval->toDt(&sinit->Sdt);
 #endif
-#ifndef IN_GCC
-        dtnbytes(&sinit->Sdt, tc->size(0), (char *)&tc->sym->defaultval);
-#else
-        Expression * e = new IntegerExp(0, tc->sym->defaultval, tc);
-        e->toDt(& sinit->Sdt);
-#endif
-        //sinit->Sdt = tc->sym->init->toDt();
         outdata(sinit);
     }
 }
 
 
+
