@@ -29,13 +29,12 @@ struct Identifier : Object
     int compare(Object *o);
     void print();
     char *toChars();
-#ifdef _DH
     char *toHChars();
-#endif
-    char *toHChars2();
+    const char *toHChars2();
     int dyncast();
 
-    static Identifier *generateId(char *prefix);
+    static Identifier *generateId(const char *prefix);
+    static Identifier *generateId(const char *prefix, size_t i);
 };
 
 #endif /* DMD_IDENTIFIER_H */
