@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -42,10 +42,12 @@ struct TypeInfoDeclaration;
 struct Dsymbol;
 struct TemplateInstance;
 struct CppMangleState;
+struct TemplateDeclaration;
 enum LINK;
 
 struct TypeBasic;
 struct HdrGenState;
+struct Parameter;
 
 // Back end
 #if IN_GCC
@@ -55,24 +57,25 @@ typedef TYPE type;
 typedef struct TYPE type;
 #endif
 struct Symbol;
+struct TypeTuple;
 
 #endif
 
 enum ENUMTY
 {
-    Tarray,		// dynamic array
-    Tsarray,		// static array
-    Taarray,		// associative array
+    Tarray,             // slice array, aka T[]
+    Tsarray,            // static array, aka T[dimension]
+    Taarray,            // associative array, aka T[type]
     Tpointer,
     Treference,
     Tfunction,
     Tident,
     Tclass,
     Tstruct,
+
     Tenum,
     Ttypedef,
     Tdelegate,
-
     Tnone,
     Tvoid,
     Tint8,
@@ -80,36 +83,34 @@ enum ENUMTY
     Tint16,
     Tuns16,
     Tint32,
+
     Tuns32,
     Tint64,
     Tuns64,
     Tfloat32,
     Tfloat64,
     Tfloat80,
-
     Timaginary32,
     Timaginary64,
     Timaginary80,
-
     Tcomplex32,
+
     Tcomplex64,
     Tcomplex80,
-
-    Tbit,
     Tbool,
     Tchar,
     Twchar,
     Tdchar,
-
     Terror,
     Tinstance,
     Ttypeof,
+
     Ttuple,
     Tslice,
     Treturn,
     TMAX
 };
-typedef unsigned char TY;	// ENUMTY
+typedef unsigned char TY;       // ENUMTY
 
 #ifndef ENUM_TY_ONLY
 
@@ -117,64 +118,80 @@ typedef unsigned char TY;	// ENUMTY
 
 extern int Tsize_t;
 extern int Tptrdiff_t;
-extern int Tindex;
 
 
 struct Type : Object
 {
     TY ty;
-    unsigned char mod;	// modifiers MODxxxx
-	#define MODconst     1	// type is const
-	#define MODinvariant 2	// type is invariant
-	#define MODshared    4	// type is shared
+    unsigned char mod;  // modifiers MODxxxx
+        /* pick this order of numbers so switch statements work better
+         */
+        #define MODconst     1  // type is const
+        #define MODimmutable 4  // type is immutable
+        #define MODshared    2  // type is shared
+        #define MODwild      8  // type is wild
+        #define MODmutable   0x10       // type is mutable (only used in wildcard matching)
     char *deco;
-    Type *cto;		// MODconst ? mutable version of this type : const version
-    Type *ito;		// MODinvariant ? mutable version of this type : invariant version
-    Type *pto;		// merged pointer to this type
-    Type *rto;		// reference to this type
-    Type *arrayof;	// array of this type
-    TypeInfoDeclaration *vtinfo;	// TypeInfo object for this Type
 
-    type *ctype;	// for back end
+    /* These are cached values that are lazily evaluated by constOf(), invariantOf(), etc.
+     * They should not be referenced by anybody but mtype.c.
+     * They can be NULL if not lazily evaluated yet.
+     * Note that there is no "shared immutable", because that is just immutable
+     * Naked == no MOD bits
+     */
 
-    #define tvoid	basic[Tvoid]
-    #define tint8	basic[Tint8]
-    #define tuns8	basic[Tuns8]
-    #define tint16	basic[Tint16]
-    #define tuns16	basic[Tuns16]
-    #define tint32	basic[Tint32]
-    #define tuns32	basic[Tuns32]
-    #define tint64	basic[Tint64]
-    #define tuns64	basic[Tuns64]
-    #define tfloat32	basic[Tfloat32]
-    #define tfloat64	basic[Tfloat64]
-    #define tfloat80	basic[Tfloat80]
+    Type *cto;          // MODconst ? naked version of this type : const version
+    Type *ito;          // MODimmutable ? naked version of this type : immutable version
+    Type *sto;          // MODshared ? naked version of this type : shared mutable version
+    Type *scto;         // MODshared|MODconst ? naked version of this type : shared const version
+    Type *wto;          // MODwild ? naked version of this type : wild version
+    Type *swto;         // MODshared|MODwild ? naked version of this type : shared wild version
+
+    Type *pto;          // merged pointer to this type
+    Type *rto;          // reference to this type
+    Type *arrayof;      // array of this type
+    TypeInfoDeclaration *vtinfo;        // TypeInfo object for this Type
+
+    type *ctype;        // for back end
+
+    #define tvoid       basic[Tvoid]
+    #define tint8       basic[Tint8]
+    #define tuns8       basic[Tuns8]
+    #define tint16      basic[Tint16]
+    #define tuns16      basic[Tuns16]
+    #define tint32      basic[Tint32]
+    #define tuns32      basic[Tuns32]
+    #define tint64      basic[Tint64]
+    #define tuns64      basic[Tuns64]
+    #define tfloat32    basic[Tfloat32]
+    #define tfloat64    basic[Tfloat64]
+    #define tfloat80    basic[Tfloat80]
 
     #define timaginary32 basic[Timaginary32]
     #define timaginary64 basic[Timaginary64]
     #define timaginary80 basic[Timaginary80]
 
-    #define tcomplex32	basic[Tcomplex32]
-    #define tcomplex64	basic[Tcomplex64]
-    #define tcomplex80	basic[Tcomplex80]
+    #define tcomplex32  basic[Tcomplex32]
+    #define tcomplex64  basic[Tcomplex64]
+    #define tcomplex80  basic[Tcomplex80]
 
-    #define tbit	basic[Tbit]
-    #define tbool	basic[Tbool]
-    #define tchar	basic[Tchar]
-    #define twchar	basic[Twchar]
-    #define tdchar	basic[Tdchar]
+    #define tbool       basic[Tbool]
+    #define tchar       basic[Tchar]
+    #define twchar      basic[Twchar]
+    #define tdchar      basic[Tdchar]
 
     // Some special types
-    #define tshiftcnt	tint32		// right side of shift expression
-//    #define tboolean	tint32		// result of boolean expression
-    #define tboolean	tbool		// result of boolean expression
-    #define tindex	basic[Tindex]	// array/ptr index
-    static Type *tvoidptr;		// void*
-    #define terror	basic[Terror]	// for error recovery
+    #define tshiftcnt   tint32          // right side of shift expression
+//    #define tboolean  tint32          // result of boolean expression
+    #define tboolean    tbool           // result of boolean expression
+    #define tindex      tsize_t         // array/ptr index
+    static Type *tvoidptr;              // void*
+    static Type *tstring;               // immutable(char)[]
+    #define terror      basic[Terror]   // for error recovery
 
-    #define tsize_t	basic[Tsize_t]		// matches size_t alias
-    #define tptrdiff_t	basic[Tptrdiff_t]	// matches ptrdiff_t alias
-    #define thash_t	tsize_t			// matches hash_t alias
+    #define tsize_t     basic[Tsize_t]          // matches size_t alias
+    #define tptrdiff_t  basic[Tptrdiff_t]       // matches ptrdiff_t alias
+    #define thash_t     tsize_t                 // matches hash_t alias
 
     static ClassDeclaration *typeinfo;
     static ClassDeclaration *typeinfoclass;
@@ -191,6 +208,10 @@ struct Type : Object
     static ClassDeclaration *typeinfotypelist;
     static ClassDeclaration *typeinfoconst;
     static ClassDeclaration *typeinfoinvariant;
+    static ClassDeclaration *typeinfoshared;
+    static ClassDeclaration *typeinfowild;
+
+    static TemplateDeclaration *associativearray;
 
     static Type *basic[TMAX];
     static unsigned char mangleChar[TMAX];
@@ -218,49 +239,74 @@ struct Type : Object
     virtual d_uns64 size(Loc loc);
     virtual unsigned alignsize();
     virtual Type *semantic(Loc loc, Scope *sc);
+    Type *trySemantic(Loc loc, Scope *sc);
     virtual void toDecoBuffer(OutBuffer *buf, int flag = 0);
     Type *merge();
+    Type *merge2();
     virtual void toCBuffer(OutBuffer *buf, Identifier *ident, HdrGenState *hgs);
     virtual void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     void toCBuffer3(OutBuffer *buf, HdrGenState *hgs, int mod);
-#if IN_GCC || TARGET_LINUX
+    void modToBuffer(OutBuffer *buf);
+#if IN_GCC || CPP_MANGLE
     virtual void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
     virtual int isintegral();
-    virtual int isfloating();	// real, imaginary, or complex
+    virtual int isfloating();   // real, imaginary, or complex
     virtual int isreal();
     virtual int isimaginary();
     virtual int iscomplex();
     virtual int isscalar();
     virtual int isunsigned();
-    virtual int isauto();
+    virtual int isscope();
     virtual int isString();
     virtual int isAssignable();
-    virtual int checkBoolean();	// if can be converted to boolean value
+    virtual int checkBoolean(); // if can be converted to boolean value
     virtual void checkDeprecated(Loc loc, Scope *sc);
-    int isConst()	{ return mod == MODconst; }
-    int isInvariant()	{ return mod == MODinvariant; }
-    int isMutable()	{ return mod == 0; }
+    int isConst()       { return mod & MODconst; }
+    int isImmutable()   { return mod & MODimmutable; }
+    int isMutable()     { return !(mod & (MODconst | MODimmutable | MODwild)); }
+    int isShared()      { return mod & MODshared; }
+    int isSharedConst() { return mod == (MODshared | MODconst); }
+    int isWild()        { return mod & MODwild; }
+    int isSharedWild()  { return mod == (MODshared | MODwild); }
+    int isNaked()       { return mod == 0; }
     Type *constOf();
     Type *invariantOf();
     Type *mutableOf();
+    Type *sharedOf();
+    Type *sharedConstOf();
+    Type *unSharedOf();
+    Type *wildOf();
+    Type *sharedWildOf();
+    void fixTo(Type *t);
+    void check();
+    Type *castMod(unsigned mod);
+    Type *addMod(unsigned mod);
+    Type *addStorageClass(StorageClass stc);
     Type *pointerTo();
     Type *referenceTo();
     Type *arrayOf();
     virtual Type *makeConst();
     virtual Type *makeInvariant();
+    virtual Type *makeShared();
+    virtual Type *makeSharedConst();
+    virtual Type *makeWild();
+    virtual Type *makeSharedWild();
+    virtual Type *makeMutable();
     virtual Dsymbol *toDsymbol(Scope *sc);
     virtual Type *toBasetype();
     virtual Type *toHeadMutable();
-    virtual int isBaseOf(Type *t, target_ptrdiff_t *poffset);
+    virtual int isBaseOf(Type *t, int *poffset);
     virtual MATCH constConv(Type *to);
     virtual MATCH implicitConvTo(Type *to);
     virtual ClassDeclaration *isClassHandle();
     virtual Expression *getProperty(Loc loc, Identifier *ident);
     virtual Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
+    Expression *noMember(Scope *sc, Expression *e, Identifier *ident);
     virtual unsigned memalign(unsigned salign);
     virtual Expression *defaultInit(Loc loc = 0);
-    virtual int isZeroInit();		// if initializer is 0
+    virtual Expression *defaultInitLiteral(Loc loc = 0);
+    virtual int isZeroInit(Loc loc = 0);                // if initializer is 0
     virtual dt_t **toDt(dt_t **pdt);
     Identifier *getTypeInfoIdent(int internal);
     virtual MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
@@ -270,11 +316,17 @@ struct Type : Object
     virtual TypeInfoDeclaration *getTypeInfoDeclaration();
     virtual int builtinTypeInfo();
     virtual Type *reliesOnTident();
+    virtual int hasWild();
+    virtual unsigned wildMatch(Type *targ);
     virtual Expression *toExpression();
     virtual int hasPointers();
+    virtual TypeTuple *toArgTypes();
     virtual Type *nextOf();
+    uinteger_t sizemask();
+    virtual int needsDestruction();
 
     static void error(Loc loc, const char *format, ...);
+    static void warning(Loc loc, const char *format, ...);
 
     // For backend
     virtual unsigned totym();
@@ -286,6 +338,19 @@ struct Type : Object
     virtual TypeBasic *isTypeBasic();
 };
 
+struct TypeError : Type
+{
+    TypeError();
+
+    void toCBuffer(OutBuffer *buf, Identifier *ident, HdrGenState *hgs);
+
+    d_uns64 size(Loc loc);
+    Expression *getProperty(Loc loc, Identifier *ident);
+    Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
+    Expression *defaultInit(Loc loc);
+    Expression *defaultInitLiteral(Loc loc);
+};
+
 struct TypeNext : Type
 {
     Type *next;
@@ -294,10 +359,18 @@ struct TypeNext : Type
     void toDecoBuffer(OutBuffer *buf, int flag);
     void checkDeprecated(Loc loc, Scope *sc);
     Type *reliesOnTident();
+    int hasWild();
+    unsigned wildMatch(Type *targ);
     Type *nextOf();
     Type *makeConst();
     Type *makeInvariant();
+    Type *makeShared();
+    Type *makeSharedConst();
+    Type *makeWild();
+    Type *makeSharedWild();
+    Type *makeMutable();
     MATCH constConv(Type *to);
+    void transitive();
 };
 
 struct TypeBasic : Type
@@ -313,11 +386,10 @@ struct TypeBasic : Type
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     char *toChars();
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
-#if IN_GCC || TARGET_LINUX
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
     int isintegral();
-    int isbit();
     int isfloating();
     int isreal();
     int isimaginary();
@@ -326,8 +398,9 @@ struct TypeBasic : Type
     int isunsigned();
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int builtinTypeInfo();
+    TypeTuple *toArgTypes();
 
     // For eliminating dynamic_cast
     TypeBasic *isTypeBasic();
@@ -354,18 +427,21 @@ struct TypeSArray : TypeArray
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     int isString();
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     unsigned memalign(unsigned salign);
     MATCH constConv(Type *to);
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
+    Expression *defaultInitLiteral(Loc loc);
     dt_t **toDt(dt_t **pdt);
     dt_t **toDtElem(dt_t **pdt, Expression *e);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     Expression *toExpression();
     int hasPointers();
-#if IN_GCC || TARGET_LINUX
+    int needsDestruction();
+    TypeTuple *toArgTypes();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -385,14 +461,16 @@ struct TypeDArray : TypeArray
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     int isString();
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
     int builtinTypeInfo();
+    MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
-#if IN_GCC || TARGET_LINUX
+    TypeTuple *toArgTypes();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -401,25 +479,31 @@ struct TypeDArray : TypeArray
 
 struct TypeAArray : TypeArray
 {
-    Type *index;		// key type
+    Type *index;                // key type
+    Loc loc;
+    Scope *sc;
+
+    StructDeclaration *impl;    // implementation
 
     TypeAArray(Type *t, Type *index);
     Type *syntaxCopy();
     d_uns64 size(Loc loc);
     Type *semantic(Loc loc, Scope *sc);
+    StructDeclaration *getImpl();
     void resolve(Loc loc, Scope *sc, Expression **pe, Type **pt, Dsymbol **ps);
     void toDecoBuffer(OutBuffer *buf, int flag);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     Expression *defaultInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+    TypeTuple *toArgTypes();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
-#if IN_GCC || TARGET_LINUX
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -437,12 +521,14 @@ struct TypePointer : TypeNext
     d_uns64 size(Loc loc);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     MATCH implicitConvTo(Type *to);
+    MATCH constConv(Type *to);
     int isscalar();
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
-#if IN_GCC || TARGET_LINUX
+    TypeTuple *toArgTypes();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -458,50 +544,77 @@ struct TypeReference : TypeNext
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
-#if IN_GCC || TARGET_LINUX
+    int isZeroInit(Loc loc);
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 };
 
 enum RET
 {
-    RETregs	= 1,	// returned in registers
-    RETstack	= 2,	// returned on stack
+    RETregs     = 1,    // returned in registers
+    RETstack    = 2,    // returned on stack
+};
+
+enum TRUST
+{
+    TRUSTdefault = 0,
+    TRUSTsystem = 1,    // @system (same as TRUSTdefault)
+    TRUSTtrusted = 2,   // @trusted
+    TRUSTsafe = 3,      // @safe
+};
+
+enum PURE
+{
+    PUREimpure = 0,     // not pure at all
+    PUREweak = 1,       // no mutable globals are read or written
+    PUREconst = 2,      // parameters are values or const
+    PUREstrong = 3,     // parameters are values or immutable
+    PUREfwdref = 4,     // it's pure, but not known which level yet
 };
 
 struct TypeFunction : TypeNext
 {
     // .next is the return type
 
-    Arguments *parameters;	// function parameters
-    int varargs;	// 1: T t, ...) style for variable number of arguments
-			// 2: T t ...) style for variable number of arguments
-    bool isnothrow;	// true: nothrow
-    bool ispure;	// true: pure
-    bool isref;		// true: returns a reference
-    enum LINK linkage;	// calling convention
+    Parameters *parameters;     // function parameters
+    int varargs;        // 1: T t, ...) style for variable number of arguments
+                        // 2: T t ...) style for variable number of arguments
+    bool isnothrow;     // true: nothrow
+    bool isproperty;    // can be called without parentheses
+    bool isref;         // true: returns a reference
+    enum LINK linkage;  // calling convention
+    enum TRUST trust;   // level of trust
+    enum PURE purity;   // PURExxxx
+    Expressions *fargs; // function arguments
 
     int inuse;
 
-    TypeFunction(Arguments *parameters, Type *treturn, int varargs, enum LINK linkage);
+    TypeFunction(Parameters *parameters, Type *treturn, int varargs, enum LINK linkage, StorageClass stc = 0);
     Type *syntaxCopy();
     Type *semantic(Loc loc, Scope *sc);
+    void purityLevel();
     void toDecoBuffer(OutBuffer *buf, int flag);
     void toCBuffer(OutBuffer *buf, Identifier *ident, HdrGenState *hgs);
+    void toCBufferWithAttributes(OutBuffer *buf, Identifier *ident, HdrGenState* hgs, TypeFunction *attrs, TemplateDeclaration *td);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
+    void attributesToCBuffer(OutBuffer *buf, int mod);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     Type *reliesOnTident();
-#if IN_GCC || TARGET_LINUX
+    bool hasLazyParameters();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
+    bool parameterEscapes(Parameter *p);
 
-    int callMatch(Expression *ethis, Expressions *toargs);
+    int callMatch(Expression *ethis, Expressions *toargs, int flag = 0);
     type *toCtype();
     enum RET retStyle();
 
     unsigned totym();
+
+    Expression *defaultInit(Loc loc);
 };
 
 struct TypeDelegate : TypeNext
@@ -512,14 +625,17 @@ struct TypeDelegate : TypeNext
     Type *syntaxCopy();
     Type *semantic(Loc loc, Scope *sc);
     d_uns64 size(Loc loc);
+    unsigned alignsize();
+    MATCH implicitConvTo(Type *to);
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     int hasPointers();
-#if IN_GCC || TARGET_LINUX
+    TypeTuple *toArgTypes();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -529,7 +645,7 @@ struct TypeDelegate : TypeNext
 struct TypeQualified : Type
 {
     Loc loc;
-    Array idents;	// array of Identifier's representing ident.ident.ident etc.
+    Identifiers idents;       // array of Identifier's representing ident.ident.ident etc.
 
     TypeQualified(TY ty, Loc loc);
     void syntaxCopyHelper(TypeQualified *t);
@@ -537,7 +653,7 @@ struct TypeQualified : Type
     void toCBuffer2Helper(OutBuffer *buf, HdrGenState *hgs);
     d_uns64 size(Loc loc);
     void resolveHelper(Loc loc, Scope *sc, Dsymbol *s, Dsymbol *scopesym,
-	Expression **pe, Type **pt, Dsymbol **ps);
+        Expression **pe, Type **pt, Dsymbol **ps);
 };
 
 struct TypeIdentifier : TypeQualified
@@ -570,12 +686,14 @@ struct TypeInstance : TypeQualified
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     void resolve(Loc loc, Scope *sc, Expression **pe, Type **pt, Dsymbol **ps);
     Type *semantic(Loc loc, Scope *sc);
+    Dsymbol *toDsymbol(Scope *sc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
 };
 
 struct TypeTypeof : TypeQualified
 {
     Expression *exp;
+    int inuse;
 
     TypeTypeof(Loc loc, Expression *exp);
     Type *syntaxCopy();
@@ -610,17 +728,20 @@ struct TypeStruct : Type
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     unsigned memalign(unsigned salign);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    Expression *defaultInitLiteral(Loc loc);
+    int isZeroInit(Loc loc);
     int isAssignable();
     int checkBoolean();
+    int needsDestruction();
     dt_t **toDt(dt_t **pdt);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+    TypeTuple *toArgTypes();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
     Type *toHeadMutable();
-#if IN_GCC || TARGET_LINUX
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -632,6 +753,7 @@ struct TypeEnum : Type
     EnumDeclaration *sym;
 
     TypeEnum(EnumDeclaration *sym);
+    Type *syntaxCopy();
     d_uns64 size(Loc loc);
     unsigned alignsize();
     char *toChars();
@@ -643,17 +765,24 @@ struct TypeEnum : Type
     Expression *getProperty(Loc loc, Identifier *ident);
     int isintegral();
     int isfloating();
+    int isreal();
+    int isimaginary();
+    int iscomplex();
     int isscalar();
     int isunsigned();
+    int checkBoolean();
+    int isAssignable();
+    int needsDestruction();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
     Type *toBasetype();
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
-#if IN_GCC || TARGET_LINUX
+    TypeTuple *toArgTypes();
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -675,7 +804,6 @@ struct TypeTypedef : Type
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     Expression *getProperty(Loc loc, Identifier *ident);
-    int isbit();
     int isintegral();
     int isfloating();
     int isreal();
@@ -685,17 +813,21 @@ struct TypeTypedef : Type
     int isunsigned();
     int checkBoolean();
     int isAssignable();
+    int needsDestruction();
     Type *toBasetype();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    Expression *defaultInitLiteral(Loc loc);
+    int isZeroInit(Loc loc);
     dt_t **toDt(dt_t **pdt);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+    TypeTuple *toArgTypes();
+    int hasWild();
     Type *toHeadMutable();
-#if IN_GCC || TARGET_LINUX
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
 #endif
 
@@ -717,19 +849,23 @@ struct TypeClass : Type
     void toCBuffer2(OutBuffer *buf, HdrGenState *hgs, int mod);
     Expression *dotExp(Scope *sc, Expression *e, Identifier *ident);
     ClassDeclaration *isClassHandle();
-    int isBaseOf(Type *t, target_ptrdiff_t *poffset);
+    int isBaseOf(Type *t, int *poffset);
     MATCH implicitConvTo(Type *to);
     Expression *defaultInit(Loc loc);
-    int isZeroInit();
+    int isZeroInit(Loc loc);
     MATCH deduceType(Scope *sc, Type *tparam, TemplateParameters *parameters, Objects *dedtypes);
-    int isauto();
+    int isscope();
     int checkBoolean();
     TypeInfoDeclaration *getTypeInfoDeclaration();
     int hasPointers();
+    TypeTuple *toArgTypes();
+    int builtinTypeInfo();
+#if DMDV2
     Type *toHeadMutable();
     MATCH constConv(Type *to);
-#if IN_GCC || TARGET_LINUX
+#if IN_GCC || CPP_MANGLE
     void toCppMangle(OutBuffer *buf, CppMangleState *cms);
+#endif
 #endif
 
     type *toCtype();
@@ -739,10 +875,13 @@ struct TypeClass : Type
 
 struct TypeTuple : Type
 {
-    Arguments *arguments;	// types making up the tuple
+    Parameters *arguments;      // types making up the tuple
 
-    TypeTuple(Arguments *arguments);
+    TypeTuple(Parameters *arguments);
     TypeTuple(Expressions *exps);
+    TypeTuple();
+    TypeTuple(Type *t1);
+    TypeTuple(Type *t1, Type *t2);
     Type *syntaxCopy();
     Type *semantic(Loc loc, Scope *sc);
     int equals(Object *o);
@@ -769,26 +908,26 @@ struct TypeSlice : TypeNext
 
 //enum InOut { None, In, Out, InOut, Lazy };
 
-struct Argument : Object
+struct Parameter : Object
 {
     //enum InOut inout;
-    unsigned storageClass;
+    StorageClass storageClass;
     Type *type;
     Identifier *ident;
     Expression *defaultArg;
 
-    Argument(unsigned storageClass, Type *type, Identifier *ident, Expression *defaultArg);
-    Argument *syntaxCopy();
+    Parameter(StorageClass storageClass, Type *type, Identifier *ident, Expression *defaultArg);
+    Parameter *syntaxCopy();
     Type *isLazyArray();
     void toDecoBuffer(OutBuffer *buf);
-    static Arguments *arraySyntaxCopy(Arguments *args);
-    static char *argsTypesToChars(Arguments *args, int varargs);
-    static void argsCppMangle(OutBuffer *buf, CppMangleState *cms, Arguments *arguments, int varargs);
-    static void argsToCBuffer(OutBuffer *buf, HdrGenState *hgs, Arguments *arguments, int varargs);
-    static void argsToDecoBuffer(OutBuffer *buf, Arguments *arguments);
-    static int isTPL(Arguments *arguments);
-    static size_t dim(Arguments *arguments);
-    static Argument *getNth(Arguments *arguments, size_t nth, size_t *pn = NULL);
+    static Parameters *arraySyntaxCopy(Parameters *args);
+    static char *argsTypesToChars(Parameters *args, int varargs);
+    static void argsCppMangle(OutBuffer *buf, CppMangleState *cms, Parameters *arguments, int varargs);
+    static void argsToCBuffer(OutBuffer *buf, HdrGenState *hgs, Parameters *arguments, int varargs);
+    static void argsToDecoBuffer(OutBuffer *buf, Parameters *arguments);
+    static int isTPL(Parameters *arguments);
+    static size_t dim(Parameters *arguments);
+    static Parameter *getNth(Parameters *arguments, size_t nth, size_t *pn = NULL);
 };
 
 extern int PTRSIZE;
@@ -797,6 +936,12 @@ extern int REALPAD;
 extern int Tsize_t;
 extern int Tptrdiff_t;
 
+int arrayTypeCompatible(Loc loc, Type *t1, Type *t2);
+int arrayTypeCompatibleWithoutCasting(Loc loc, Type *t1, Type *t2);
+void MODtoBuffer(OutBuffer *buf, unsigned char mod);
+int MODimplicitConv(unsigned char modfrom, unsigned char modto);
+int MODmerge(unsigned char mod1, unsigned char mod2);
+
 #endif
 
 #endif /* DMD_MTYPE_H */
